@@ -4,6 +4,7 @@ from __future__ import annotations
 
 import asyncio
 import concurrent.futures
+import contextvars
 import inspect
 import logging
 import multiprocessing
@@ -16,7 +17,18 @@ from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from dataclasses import dataclass
 from datetime import datetime, timedelta, timezone
-from typing import Any, Callable, Dict, Iterator, Optional, Sequence, Tuple, Type, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterator,
+    NoReturn,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    Union,
+)
 
 import google.protobuf.duration_pb2
 import google.protobuf.timestamp_pb2
@@ -64,6 +76,7 @@ class _ActivityWorker:
         self._running_activities: Dict[bytes, _RunningActivity] = {}
         self._data_converter = data_converter
         self._interceptors = interceptors
+        self._fail_worker_exception_queue: asyncio.Queue[Exception] = asyncio.Queue()
         # Lazily created on first activity
         self._worker_shutdown_event: Optional[
             temporalio.activity._CompositeEvent
@@ -79,6 +92,12 @@ class _ActivityWorker:
             if defn.name in self._activities:
                 raise ValueError(f"More than one activity named {defn.name}")
 
+            # Do not allow classes, __call__ based activities must be instances
+            if inspect.isclass(activity):
+                raise TypeError(
+                    f"Activity named {defn.name} is a class instead of an instance"
+                )
+
             # Some extra requirements for sync functions
             if not defn.is_async:
                 if not activity_executor:
@@ -111,11 +130,26 @@ class _ActivityWorker:
             self._activities[defn.name] = defn
 
     async def run(self) -> None:
+        # Create a task that fails when we get a failure on the queue
+        async def raise_from_queue() -> NoReturn:
+            raise await self._fail_worker_exception_queue.get()
+
+        exception_task = asyncio.create_task(raise_from_queue())
+
         # Continually poll for activity work
         while True:
             try:
                 # Poll for a task
-                task = await self._bridge_worker().poll_activity_task()
+                poll_task = asyncio.create_task(
+                    self._bridge_worker().poll_activity_task()
+                )
+                await asyncio.wait([poll_task, exception_task], return_when=asyncio.FIRST_COMPLETED)  # type: ignore
+                # If exception for failing the worker happened, raise it.
+                # Otherwise, the poll succeeded.
+                if exception_task.done():
+                    poll_task.cancel()
+                    await exception_task
+                task = await poll_task
 
                 if task.HasField("start"):
                     # Cancelled event and sync field will be updated inside
@@ -131,8 +165,10 @@ class _ActivityWorker:
                 else:
                     raise RuntimeError(f"Unrecognized activity task: {task}")
             except temporalio.bridge.worker.PollShutdownError:
+                exception_task.cancel()
                 return
             except Exception as err:
+                exception_task.cancel()
                 raise RuntimeError("Activity worker failed") from err
 
     async def shutdown(self, after_graceful_timeout: timedelta) -> None:
@@ -465,6 +501,10 @@ class _ActivityWorker:
                     await self._data_converter.encode_failure(
                         err, completion.result.failed.failure
                     )
+
+                    # For broken executors, we have to fail the entire worker
+                    if isinstance(err, concurrent.futures.BrokenExecutor):
+                        self._fail_worker_exception_queue.put_nowait(err)
             except Exception as inner_err:
                 temporalio.activity.logger.exception(
                     f"Exception handling failed, original error: {err}"
@@ -635,9 +675,9 @@ class _ActivityInboundImpl(ActivityInboundInterceptor):
                 assert cancelled_event
                 worker_shutdown_event = self._worker._worker_shutdown_event
                 assert worker_shutdown_event
-                return await loop.run_in_executor(
-                    input.executor,
-                    _execute_sync_activity,
+                # Prepare func and args
+                func: Callable = _execute_sync_activity
+                args = [
                     info,
                     heartbeat,
                     self._running_activity.cancel_thread_raiser,
@@ -646,7 +686,16 @@ class _ActivityInboundImpl(ActivityInboundInterceptor):
                     worker_shutdown_event.thread_event,
                     input.fn,
                     *input.args,
-                )
+                ]
+                # If we're threaded, we want to pass the context through. We
+                # have to do this manually, see
+                # https://github.com/python/cpython/issues/78195.
+                if isinstance(input.executor, concurrent.futures.ThreadPoolExecutor):
+                    current_context = contextvars.copy_context()
+                    args.insert(0, func)
+                    func = current_context.run
+                # Invoke
+                return await loop.run_in_executor(input.executor, func, *args)
             finally:
                 if shared_manager:
                     await shared_manager.unregister_heartbeater(info.task_token)
